@@ -83,6 +83,8 @@ GAIL_ACCESSIBLE_FACTORY (GAIL_TYPE_PANED, gail_paned, gail_paned_new)
 GAIL_ACCESSIBLE_FACTORY (GAIL_TYPE_SCROLLBAR, gail_scrollbar, gail_scrollbar_new)
 GAIL_ACCESSIBLE_FACTORY (GAIL_TYPE_OPTION_MENU, gail_option_menu, gail_option_menu_new)
 GAIL_ACCESSIBLE_FACTORY (GAIL_TYPE_CANVAS, gail_canvas, gail_canvas_new)
+GAIL_ACCESSIBLE_FACTORY (GAIL_TYPE_CHECK_MENU_ITEM, gail_check_menu_item, gail_check_menu_item_new)
+GAIL_ACCESSIBLE_FACTORY (GAIL_TYPE_RADIO_MENU_ITEM, gail_radio_menu_item, gail_radio_menu_item_new)
 
 static AtkObject*
 gail_get_accessible_for_widget (GtkWidget *widget,
@@ -91,6 +93,9 @@ gail_get_accessible_for_widget (GtkWidget *widget,
   AtkObject *obj = NULL;
 
   *transient = FALSE;
+  if (!widget)
+    return NULL;
+
   if (GTK_IS_ENTRY (widget))
     {
       GtkWidget *other_widget = widget->parent;
@@ -102,7 +107,7 @@ gail_get_accessible_for_widget (GtkWidget *widget,
       GtkNotebook *notebook;
       gint page_num;
 
-      notebook = GTK_WIDGET (widget);
+      notebook = GTK_NOTEBOOK (widget);
       /*
        * Report the currently focused tab rather than the currently selected tab
        */
@@ -174,18 +179,7 @@ gail_focus_watcher (GSignalInvocationHint *ihint,
               window = GTK_WINDOW (widget);
               if (window->focus_widget)
                 {
-                  /*
-                   * Found focus widget so use that unless it is already the
-                   * focus widget
-                   */
-                  if (focus_widget == window->focus_widget)
-                    {
-                      return TRUE;
-                    }
-                  else
-                    {
-                      widget = window->focus_widget;
-                    }
+                  widget = window->focus_widget;
                 }
               else if (window->type == GTK_WINDOW_POPUP)
                 {
@@ -211,8 +205,8 @@ gail_focus_watcher (GSignalInvocationHint *ihint,
         }
       else
         {
-          /* Ignore focus out */
-          return TRUE;
+          /* focus out */
+          widget = NULL;
         }
     }
   else
@@ -335,7 +329,7 @@ static gint
 gail_focus_idle_handler (gpointer data)
 {
   focus_notify_handler = 0;
-  gail_focus_notify (GTK_WIDGET (data));
+  gail_focus_notify (data);
 
   return FALSE; 
 }
@@ -353,23 +347,29 @@ gail_focus_notify (GtkWidget *widget)
           g_object_remove_weak_pointer (G_OBJECT (focus_widget), (gpointer *)&focus_widget);
         }
       focus_widget = widget;
-      g_object_add_weak_pointer (G_OBJECT (focus_widget), (gpointer *)&focus_widget);
-      /*
-       * The UI may not have been updated yet; e.g. in gtkhtml2
-       * html_view_layout() is called in a idle handler
-       */
-      if (focus_widget == focus_before_menu)
+      if (focus_widget)
         {
-          g_object_remove_weak_pointer (G_OBJECT (focus_before_menu), (gpointer *)&focus_before_menu);
-          focus_before_menu = NULL;
+          g_object_add_weak_pointer (G_OBJECT (focus_widget), (gpointer *)&focus_widget);
+          /*
+           * The UI may not have been updated yet; e.g. in gtkhtml2
+           * html_view_layout() is called in a idle handler
+           */
+          if (focus_widget == focus_before_menu)
+            {
+              g_object_remove_weak_pointer (G_OBJECT (focus_before_menu), (gpointer *)&focus_before_menu);
+              focus_before_menu = NULL;
+            }
         }
       gail_focus_notify_when_idle (focus_widget);
     }
   else
     {
-      atk_obj  = gail_get_accessible_for_widget (focus_widget, &transient);
+      if (focus_widget)
+        atk_obj  = gail_get_accessible_for_widget (focus_widget, &transient);
+      else
+        atk_obj = NULL;
       atk_focus_tracker_notify (atk_obj);
-      if (transient)
+      if (atk_obj && transient)
         g_object_unref (atk_obj);
     }
 }
@@ -382,14 +382,7 @@ gail_focus_notify_when_idle (GtkWidget *widget)
       gtk_idle_remove (focus_notify_handler);
     }
 
-  if (widget)
-    {
-      focus_notify_handler = gtk_idle_add (gail_focus_idle_handler, widget);
-    }
-  else
-    {
-      focus_notify_handler = 0;
-    }
+  focus_notify_handler = gtk_idle_add (gail_focus_idle_handler, widget);
 }
 static gboolean
 gail_deactivate_watcher (GSignalInvocationHint *ihint,
@@ -524,6 +517,8 @@ gail_accessibility_module_init (void)
   GAIL_WIDGET_SET_FACTORY (GTK_TYPE_PANED, gail_paned);
   GAIL_WIDGET_SET_FACTORY (GTK_TYPE_SCROLLBAR, gail_scrollbar);
   GAIL_WIDGET_SET_FACTORY (GTK_TYPE_OPTION_MENU, gail_option_menu);
+  GAIL_WIDGET_SET_FACTORY (GTK_TYPE_CHECK_MENU_ITEM, gail_check_menu_item);
+  GAIL_WIDGET_SET_FACTORY (GTK_TYPE_RADIO_MENU_ITEM, gail_radio_menu_item);
 
   /* LIBGNOMECANVAS SUPPORT */
   GAIL_WIDGET_SET_FACTORY (GNOME_TYPE_CANVAS, gail_canvas);
