@@ -60,6 +60,11 @@ static gboolean              gail_button_set_description(AtkAction      *action,
 static void                  atk_image_interface_init   (AtkImageIface  *iface);
 static G_CONST_RETURN gchar* gail_button_get_image_description 
                                                         (AtkImage       *image);
+static void	             gail_button_get_image_position
+                                                        (AtkImage       *image,
+                                                         gint	        *x,
+                                                         gint	        *y,
+                                                         AtkCoordType   coord_type);
 static void                  gail_button_get_image_size (AtkImage       *image,
                                                          gint           *width,
                                                          gint           *height);
@@ -568,6 +573,7 @@ atk_image_interface_init (AtkImageIface *iface)
   g_return_if_fail (iface != NULL);
 
   iface->get_image_description = gail_button_get_image_description;
+  iface->get_image_position = gail_button_get_image_position;
   iface->get_image_size = gail_button_get_image_size;
   iface->set_image_description = gail_button_set_image_description;
 }
@@ -621,6 +627,42 @@ gail_button_get_image_description (AtkImage *image) {
     return NULL;
 }
 
+static void
+gail_button_get_image_position (AtkImage     *image,
+                                gint	     *x,
+                                gint	     *y,
+                                AtkCoordType coord_type)
+{
+  GtkWidget *widget;
+  GtkImage  *button_image;
+  AtkObject *obj;
+
+  widget = GTK_ACCESSIBLE (image)->widget;
+
+  if (widget == NULL)
+    {
+    /*
+     * State is defunct
+     */
+      *x = G_MININT;
+      *y = G_MININT;
+      return;
+    }
+
+  button_image = get_image_from_button (widget);
+
+  if (button_image != NULL)
+    {
+      obj = gtk_widget_get_accessible (GTK_WIDGET (button_image));
+      atk_component_get_position (ATK_COMPONENT (obj), x, y, coord_type); 
+    }
+  else
+    {
+      *x = G_MININT;
+      *y = G_MININT;
+    }
+}
+
 static void
 gail_button_get_image_size (AtkImage *image,
                             gint     *width,
