@@ -115,7 +115,7 @@ gail_item_get_name (AtkObject *obj)
       if (!GTK_IS_CONTAINER (widget))
         return NULL;
     
-      children = gtk_container_children (GTK_CONTAINER (widget));
+      children = gtk_container_get_children (GTK_CONTAINER (widget));
       tmp_list = children;
       while (tmp_list != NULL)
         {
