@@ -149,11 +149,6 @@ gail_menu_item_new (GtkWidget *widget)
 
   if (GTK_IS_TEAROFF_MENU_ITEM (widget))
     accessible->role = ATK_ROLE_TEAR_OFF_MENU_ITEM;
-  else if (GTK_IS_CHECK_MENU_ITEM (widget))
-    if (GTK_IS_RADIO_MENU_ITEM (widget))
-      accessible->role = ATK_ROLE_RADIO_MENU_ITEM;
-    else
-      accessible->role = ATK_ROLE_CHECK_MENU_ITEM;
   else if (GTK_IS_SEPARATOR_MENU_ITEM (widget))
     accessible->role = ATK_ROLE_SEPARATOR;
   if (gtk_menu_item_get_submenu (GTK_MENU_ITEM(widget)))
@@ -161,6 +156,9 @@ gail_menu_item_new (GtkWidget *widget)
   else
     accessible->role = ATK_ROLE_MENU_ITEM;
 
+  g_object_set_data (G_OBJECT (accessible), "atk-component-layer",
+                     GINT_TO_POINTER (ATK_LAYER_POPUP));
+
   return accessible;
 }
 
@@ -189,7 +187,7 @@ gail_menu_item_get_n_children (AtkObject* obj)
 
 static AtkObject*
 gail_menu_item_ref_child (AtkObject *obj,
-                          gint      i)
+                          gint       i)
 {
   AtkObject  *accessible;
   GtkWidget *widget;
@@ -201,7 +199,7 @@ gail_menu_item_ref_child (AtkObject *obj,
   if (widget == NULL)
     return NULL;
 
-  if (GTK_MENU_ITEM (widget)->submenu)
+  if (GTK_MENU_ITEM (widget)->submenu && i == 0)
     {
       accessible = gtk_widget_get_accessible (GTK_MENU_ITEM (widget)->submenu);
       g_object_ref (accessible);
@@ -256,6 +254,9 @@ gail_menu_item_do_action (AtkAction *action,
         /* State is defunct */
         return FALSE;
 
+      if (!GTK_WIDGET_SENSITIVE (item) || !GTK_WIDGET_VISIBLE (item))
+        return FALSE;
+
       gtk_menu_shell_select_item (GTK_MENU_SHELL (item->parent), item);
       /*
        * This is what is called when <Return> is pressed for a menu item
@@ -340,8 +341,9 @@ gail_menu_item_get_keybinding (AtkAction *action,
           child = gtk_bin_get_child (GTK_BIN (temp_item));
           if (child == NULL)
             {
-              /* Probably a tear off menu item */
-              g_return_val_if_fail (GTK_IS_TEAROFF_MENU_ITEM (temp_item), NULL);
+              /* Possibly a tear off menu item; it could also be a menu 
+               * separator generated by gtk_item_factory_create_items()
+               */
               return NULL;
             }
           parent = gtk_widget_get_parent (temp_item);
